/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2002 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 2.02 of the PHP license,      |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/2_02.txt.                                 |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author: Tal Peer <tal@php.net>                                       |
  +----------------------------------------------------------------------+

  $Id: netools.c,v 1.5 2002/12/12 12:12:09 tal Exp $ 
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_netools.h"
#include <lcrzo.h>

/*
ZEND_DECLARE_MODULE_GLOBALS(netools)
*/

/* {{{ netools_functions[] */
/* only general functions here, not class methods */
function_entry netools_functions[] = {
	PHP_FE(netools_device_enum, NULL)
	{NULL, NULL, NULL}
};
/* }}} */

/* {{{ netools_device_class_functions[] */
static zend_class_entry *netools_device_class_entry_ptr;

static zend_function_entry netools_device_class_functions[] = {
	PHP_FE(netools_device, NULL)
	PHP_NAMED_FE(get_mtu, netools_device_get_mtu, NULL)
	PHP_NAMED_FE(get_hostname, netools_device_get_hostname, NULL)
	PHP_NAMED_FE(get_ip, netools_device_get_ip, NULL)
	PHP_NAMED_FE(get_mask, netools_device_get_mask, NULL)
	PHP_NAMED_FE(get_mac, netools_device_get_mac, NULL)
	{NULL, NULL, NULL}
};
/* }}} */

/* {{{ netools_module_entry
 */
zend_module_entry netools_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"netools",
	netools_functions, 
	PHP_MINIT(netools),
	PHP_MSHUTDOWN(netools),
	NULL,
	NULL,
	PHP_MINFO(netools),
#if ZEND_MODULE_API_NO >= 20010901
	"0.1", /* Replace with version number for your extension */
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_NETOOLS
ZEND_GET_MODULE(netools)
#endif

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(netools)
{
	zend_class_entry netools_device_class_entry;
	
	lcrzo_init();
	INIT_CLASS_ENTRY(netools_device_class_entry, "netools_device", netools_device_class_functions);
	
	zend_register_internal_class(&netools_device_class_entry TSRMLS_CC);
	
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(netools)
{
	lcrzo_close();
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(netools)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "netools support", "enabled");
	php_info_print_table_end();
}
/* }}} */

/* {{{ proto array netools_device_enum()
*/
PHP_FUNCTION(netools_device_enum)
{
	lcrzo_int32 dev_count, pos = 1;;
	
	if (ZEND_NUM_ARGS() != 0) {
		WRONG_PARAM_COUNT;
	}

	lcrzo_device_count(LCRZO_DEVICE_TYPE_ALL, &dev_count);
	array_init(return_value);
	
	while (pos <= dev_count) {
		lcrzo_device dev;

		lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_ALL, pos, dev);
		add_next_index_string(return_value, (char *)dev, 1);
		pos++;
	}	
}
/* }}} */

/* {{{ proto object netools_device(string devicename)
    */
PHP_FUNCTION(netools_device)
{
	char *devicename = NULL;
	int argc = ZEND_NUM_ARGS();
	int devicename_len, err;
	lcrzo_device device;
	lcrzo_uint32 mtu;
	lcrzo_ipl ipl;
	lcrzo_ipl maskl;
	lcrzo_ips ip;
	lcrzo_ips mask;
	lcrzo_hs hostname;
	lcrzo_etha etha;
	lcrzo_eths eths;
	

	if (zend_parse_parameters(argc TSRMLS_CC, "s", &devicename, &devicename_len) == FAILURE) 
		return;
	
	/* prevent a buffer overflow */
	if (devicename_len > LCRZO_DEVICE_MAXBYTES) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Device string is too long.");
	} else {
		err = lcrzo_device_init(devicename, device);
		lcrzo_mtu_init_device(device, &mtu);
		lcrzo_ipl_init_device(device, &ipl, &maskl);
		lcrzo_ips_init_ipl(ipl, ip);
		lcrzo_ips_init_ipl(maskl, mask);
		lcrzo_hs_init_device(device, hostname);
		if (err != LCRZO_ERR_SPDEVNOTETHER) {
			lcrzo_etha_init_device(device, etha);
			lcrzo_eths_init_etha(etha, eths);
			add_property_string(this_ptr, "mac", eths, 1);
		}
	
		add_property_string(this_ptr, "device", devicename, 1);
		add_property_long(this_ptr, "mtu", mtu);
		add_property_string(this_ptr, "ip", ip, 1);
		add_property_string(this_ptr, "mask", mask, 1);
		add_property_string(this_ptr, "hostname", hostname, 1);
	}
}
/* }}} */

/* {{{ proto int netools_device_get_mtu()
    */
PHP_NAMED_FUNCTION(netools_device_get_mtu)
{	
	zval **mtu;
	
	if (ZEND_NUM_ARGS() != 0) {
		WRONG_PARAM_COUNT;
	}
	
	if (zend_hash_find(Z_OBJPROP_P(this_ptr), "mtu", 4, (void **) &mtu) != FAILURE) {
		RETURN_LONG(Z_LVAL_PP(mtu));
	} else {
		RETURN_NULL();
	}
}
/* }}} */

/* {{{ proto string netools_device_get_hostname()
    */
PHP_NAMED_FUNCTION(netools_device_get_hostname)
{
	zval **hostname;
	
	if (ZEND_NUM_ARGS() != 0) {
		WRONG_PARAM_COUNT;
	}
	
	if (zend_hash_find(Z_OBJPROP_P(this_ptr), "hostname", 9, (void **) &hostname) != FAILURE) {
		RETURN_STRING(Z_STRVAL_PP(hostname), 1);
	} else {
		RETURN_NULL();
	}
}
/* }}} */

/* {{{ proto mixed netools_device_get_ip([bool arr])
    */
PHP_NAMED_FUNCTION(netools_device_get_ip)
{
	int argc = ZEND_NUM_ARGS();
	zend_bool arr = 0;
	char *ret = NULL;
	zval **ip;

	if (zend_parse_parameters(argc TSRMLS_CC, "|b", &arr) == FAILURE) 
		return;

	if (zend_hash_find(Z_OBJPROP_P(this_ptr), "ip", 3, (void **) &ip) != FAILURE) {
		if (!arr) {
			RETURN_STRING(Z_STRVAL_PP(ip), 1);
		} else {
			lcrzo_ipl ipl;
			
			lcrzo_ipl_init_ips(Z_STRVAL_PP(ip), &ipl);
			array_init(return_value);
			add_next_index_long(return_value, (lcrzo_uint8)(ipl>>24));
			add_next_index_long(return_value, (lcrzo_uint8)((ipl>>16)&0xFF));
			add_next_index_long(return_value, (lcrzo_uint8)((ipl>>8)&0xFF));
			add_next_index_long(return_value, (lcrzo_uint8)(ipl&0xFF));
		}
	} else {
		RETURN_NULL();
	}
}
/* }}} */

/* {{{ proto mixed netools_device_get_mask([bool arr])
*/
PHP_NAMED_FUNCTION(netools_device_get_mask)
{
	int argc = ZEND_NUM_ARGS();
	zend_bool arr = 0;
	zval **mask;

	if (zend_parse_parameters(argc TSRMLS_CC, "|b", &arr) == FAILURE) {
		return;
	}

	if (zend_hash_find(Z_OBJPROP_P(this_ptr), "mask", 5, (void **) &mask) != FAILURE) {
		if (!arr) {
			RETURN_STRING(Z_STRVAL_PP(mask), 1);
		} else {
			lcrzo_ipl maskl;
			
			lcrzo_ipl_init_ips(Z_STRVAL_PP(mask), &maskl);
			array_init(return_value);
			add_next_index_long(return_value, (lcrzo_uint8)(maskl>>24));
			add_next_index_long(return_value, (lcrzo_uint8)((maskl>>16)&0xFF));
			add_next_index_long(return_value, (lcrzo_uint8)((maskl>>8)&0xFF));
			add_next_index_long(return_value, (lcrzo_uint8)(maskl&0xFF));
		}
	} else {
		RETURN_NULL();
	}
}
/* }}} */

/* {{{ proto mixed netools_device_get_mac([bool arr])
*/
PHP_NAMED_FUNCTION(netools_device_get_mac)
{
	int argc = ZEND_NUM_ARGS();
	zend_bool arr = 0;
	zval **mac;

	if (zend_parse_parameters(argc TSRMLS_CC, "|b", &arr) == FAILURE) {
		return;
	}
	if (zend_hash_find(Z_OBJPROP_P(this_ptr), "mac", 4, (void **) &mac) != FAILURE) {
		if (!arr) {
			RETURN_STRING(Z_STRVAL_PP(mac), 1);
		} else {
			lcrzo_etha etha;
			int i = 0;
			char *hex[6];
			
			lcrzo_etha_init_eths(Z_STRVAL_PP(mac), etha);

			while (i <= 5) {
				spprintf(&hex[i], 2, "%02X", etha[i]);
				i++;
			}

			array_init(return_value);
			add_next_index_string(return_value, hex[0], 0);
			add_next_index_string(return_value, hex[1], 0);
			add_next_index_string(return_value, hex[2], 0);
			add_next_index_string(return_value, hex[3], 0);
			add_next_index_string(return_value, hex[4], 0);
			add_next_index_string(return_value, hex[5], 0);
		}
	} else {
		RETURN_NULL();
	}
}
/* }}} */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
